package binarytree.gui;

import java.util.ArrayList;
import java.util.HashMap;

import binarytree.core.BinarySearchTree;
import binarytree.core.BinaryTree;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;

public class BinaryTreeController {

	@FXML
	TextField insertValue;

	@FXML
	Pane visualTree;

	ArrayList<Node> treeNodes;
	ArrayList<Node> treeEdges;

	@FXML
	Button clear;

	BinarySearchTree<String> root = new BinarySearchTree<>();

	final double textPadding = 5;

	@FXML
	void initialize() {
		clear.setOnAction(event -> {
			for (String s : root.inorder()) {
				root.remove(s);
			}
			redrawTree();
		});
	}

	@FXML
	void insert() {
		if (insertValue.getText().length() > 0) {
			root.insert(insertValue.getText());
			redrawTree();
			insertValue.setText("");
		}
	}

	void redrawTree() {
		visualTree.getChildren().clear();

		// Collect nodes and edges in lists, so we can draw all edges before we
		// draw all nodes on top.
		treeNodes = new ArrayList<>();
		treeEdges = new ArrayList<>();

		// ArrayList<ArrayList<ArrayList<String>>> levels = root.levelOrder();
		double yOffset = visualTree.getHeight() / (root.height() + 2);

		if (!root.isEmpty())
			drawTree(root.getTree(), 0, visualTree.getWidth(), yOffset, yOffset);

		for (Node e : treeEdges) visualTree.getChildren().add(e);
		for (Node n : treeNodes) visualTree.getChildren().add(n);
	}

	// Draw a non-empty binary tree. The tree will be drawn entirely between the
	// x-coordinates xmin and xmax, with the root at y-coordinate y, and each
	// subsequent layer yStep units below that.
	//
	// Precondition: tree is not empty.
	private void drawTree(BinaryTree<String> tree, double xmin, double xmax, double y, double yStep) {

		// Make a Text node for the root, at (mid, y), where mid is the midpoint
		// between xmin and xmax.
		Text nodeText = new Text(tree.getData());
		double mid = (xmin + xmax) / 2;
		double textWidth = nodeText.getLayoutBounds().getWidth();
		double textHeight = nodeText.getLayoutBounds().getHeight();
		nodeText.setX(mid - textWidth / 2);
		nodeText.setY(y + textHeight / 2);

		Rectangle textBox = new Rectangle(mid - textWidth / 2 - textPadding, y - textHeight / 2 - textPadding,
				textWidth + 2 * textPadding, textHeight + 2 * textPadding);
		textBox.setFill(Color.WHITE);
		textBox.setStroke(Color.BLACK);
		treeNodes.add(textBox);
		treeNodes.add(nodeText);

		// Delete the node when we click on it.
		textBox.setOnMouseClicked(event -> {
			root.remove(nodeText.getText());
			redrawTree();
		});
		nodeText.setOnMouseClicked(event -> {
			root.remove(nodeText.getText());
			redrawTree();
		});

		// If there is a left subtree, draw an edge to its root and draw it
		// recursively.
		if (tree.hasLeft()) {
			Line connection = new Line(mid, y, (xmin + mid) / 2, y + yStep);
			treeEdges.add(connection);

			drawTree(tree.getLeft(), xmin, mid, y + yStep, yStep);
		}

		// Similarly for a right subtree.
		if (tree.hasRight()) {
			Line connection = new Line(mid, y, (mid + xmax) / 2, y + yStep);
			visualTree.getChildren().add(connection);

			drawTree(tree.getRight(), mid, xmax, y + yStep, yStep);
		}
	}
}
